@@ -33,16 +33,17 @@ class ClientTest(testtools.TestCase):
 
         self.assertEqual(self.conn, sc.conn)
         self.assertEqual(self.service, sc.service)
-        mock_conn.assert_called_once_with(None, None)
+        mock_conn.assert_called_once_with(prof=None, user_agent=None)
 
     def test_init_with_params(self, mock_conn):
         mock_conn.return_value = self.conn
 
-        sc = client.Client(preferences='FOO', user_agent='BAR', zoo='LARR')
+        sc = client.Client(prof='FOO', user_agent='BAR', zoo='LARR')
 
         self.assertEqual(self.conn, sc.conn)
         self.assertEqual(self.service, sc.service)
-        mock_conn.assert_called_once_with('FOO', 'BAR', zoo='LARR')
+        mock_conn.assert_called_once_with(prof='FOO', user_agent='BAR',
+                                          zoo='LARR')
 
     def test_profile_types(self, mock_conn):
         mock_conn.return_value = self.conn
@@ -304,6 +305,23 @@ class ClientTest(testtools.TestCase):
         self.service.cluster_update_policy.assert_called_once_with(
             'FOO', 'BAR', foo='bar')
 
+    def test_check_cluster(self, mock_conn):
+        mock_conn.return_value = self.conn
+        sc = client.Client()
+
+        res = sc.check_cluster('FAKE_CLUSTER_ID')
+        self.assertEqual(self.service.check_cluster.return_value, res)
+        self.service.check_cluster.assert_called_once_with('FAKE_CLUSTER_ID')
+
+    def test_recover_cluster(self, mock_conn):
+        mock_conn.return_value = self.conn
+        sc = client.Client()
+
+        res = sc.recover_cluster('FAKE_CLUSTER_ID')
+        self.assertEqual(self.service.recover_cluster.return_value, res)
+        self.service.recover_cluster.assert_called_once_with(
+            'FAKE_CLUSTER_ID')
+
     def test_nodes(self, mock_conn):
         mock_conn.return_value = self.conn
         sc = client.Client()
@@ -318,16 +336,16 @@ class ClientTest(testtools.TestCase):
 
         res = sc.get_node('FOOBAR')
         self.assertEqual(self.service.get_node.return_value, res)
-        self.service.get_node.assert_called_once_with('FOOBAR', args=None)
+        self.service.get_node.assert_called_once_with('FOOBAR', details=False)
 
     def test_get_node_with_details(self, mock_conn):
         mock_conn.return_value = self.conn
         sc = client.Client()
 
-        res = sc.get_node('FOOBAR', args={'show_details': True})
+        res = sc.get_node('FOOBAR', details=True)
         self.assertEqual(self.service.get_node.return_value, res)
         self.service.get_node.assert_called_once_with(
-            'FOOBAR', args={'show_details': True})
+            'FOOBAR', details=True)
 
     def test_create_node(self, mock_conn):
         mock_conn.return_value = self.conn
@@ -356,6 +374,23 @@ class ClientTest(testtools.TestCase):
         self.service.delete_node.assert_called_once_with(
             'FAKE_ID', True)
 
+    def test_check_node(self, mock_conn):
+        mock_conn.return_value = self.conn
+        sc = client.Client()
+
+        res = sc.check_node('FAKE_ID')
+        self.assertEqual(self.service.check_node.return_value, res)
+        self.service.check_node.assert_called_once_with('FAKE_ID')
+
+    def test_recover_node(self, mock_conn):
+        mock_conn.return_value = self.conn
+        sc = client.Client()
+
+        res = sc.recover_node('FAKE_ID')
+        self.assertEqual(self.service.recover_node.return_value, res)
+        self.service.recover_node.assert_called_once_with(
+            'FAKE_ID')
+
     def test_delete_node_ignore_missing(self, mock_conn):
         mock_conn.return_value = self.conn
         sc = client.Client()
